@@ -8,6 +8,7 @@ from typing import (
     Sequence,
     Set,
     Mapping,
+    Union,
 )
 
 from icontract import ensure
@@ -159,12 +160,10 @@ def _transpile_pattern_verification(
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            f"""\
-        private static Regex {construct_name}()
-        {{
-        """
-        )
+        f"""\
+private static Regex {construct_name}()
+{{
+"""
     )
 
     defined_variables = set()  # type: Set[Identifier]
@@ -256,13 +255,11 @@ def _transpile_pattern_verification(
     method_name = csharp_naming.method_name(verification.name)
 
     writer.write(
-        textwrap.dedent(
-            f"""\
-            public static bool {method_name}(string {arg_name})
-            {{
-            {I}return {regex_name}.IsMatch({arg_name});
-            }}"""
-        )
+        f"""\
+public static bool {method_name}(string {arg_name})
+{{
+{I}return {regex_name}.IsMatch({arg_name});
+}}"""
     )
 
     blocks.append(Stripped(writer.getvalue()))
@@ -888,8 +885,8 @@ class _InvariantTranspiler(
 
         return Stripped(writer.getvalue()), None
 
-    def transform_all(
-        self, node: parse_tree.All
+    def _transform_any_or_all(
+        self, node: Union[parse_tree.Any, parse_tree.All]
     ) -> Tuple[Optional[Stripped], Optional[Error]]:
         errors = []  # type: List[Error]
 
@@ -926,18 +923,33 @@ class _InvariantTranspiler(
         ):
             iteration = Stripped(f"({iteration})")
 
-        # NOTE (mristin, 2022-04-07):
-        # We can not use ``textwrap.dedent`` here since ``condition`` contains multiple
-        # lines.
+        qualifier_function = None  # type: Optional[str]
+        if isinstance(node, parse_tree.Any):
+            qualifier_function = "Any"
+        elif isinstance(node, parse_tree.All):
+            qualifier_function = "All"
+        else:
+            assert_never(node)
+
         return (
             Stripped(
                 f"""\
-{iteration}.All(
+{iteration}.{qualifier_function}(
 {I}{variable} => {indent_but_first_line(condition, II)})"""
             ),
             None,
         )
 
+    def transform_any(
+        self, node: parse_tree.Any
+    ) -> Tuple[Optional[Stripped], Optional[Error]]:
+        return self._transform_any_or_all(node)
+
+    def transform_all(
+        self, node: parse_tree.All
+    ) -> Tuple[Optional[Stripped], Optional[Error]]:
+        return self._transform_any_or_all(node)
+
 
 # noinspection PyProtectedMember,PyProtectedMember
 assert all(
@@ -1085,12 +1097,10 @@ def _transpile_invariant(
 
     writer.write(
         textwrap.indent(
-            textwrap.dedent(
-                f"""\
-                yield return new Reporting.Error(
-                {I}"Invariant violated:\\n" +
-                """
-            ),
+            f"""\
+yield return new Reporting.Error(
+{I}"Invariant violated:\\n" +
+""",
             I,
         )
     )
@@ -1163,15 +1173,13 @@ def _generate_enum_value_sets(symbol_table: intermediate.SymbolTable) -> Strippe
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-        /// <summary>
-        /// Hash allowed enum values for efficient validation of enums.
-        /// </summary>
-        internal static class EnumValueSet
-        {
-        """
-        )
+        """\
+/// <summary>
+/// Hash allowed enum values for efficient validation of enums.
+/// </summary>
+internal static class EnumValueSet
+{
+"""
     )
     for i, block in enumerate(blocks):
         if i > 0:
@@ -1463,23 +1471,19 @@ def _generate_transform_for_class(
     if len(blocks) == 0:
         blocks.append(
             Stripped(
-                textwrap.dedent(
-                    f"""\
-                    // No verification has been defined for {name}.
-                    yield break;"""
-                )
+                f"""\
+// No verification has been defined for {name}.
+yield break;"""
             )
         )
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            f"""\
-            public override IEnumerable<Reporting.Error> Transform(
-            {I}Aas.{name} that)
-            {{
-            """
-        )
+        f"""\
+public override IEnumerable<Reporting.Error> Transform(
+{I}Aas.{name} that)
+{{
+"""
     )
 
     for i, stmt in enumerate(blocks):
@@ -1552,13 +1556,11 @@ def _generate_transformer(
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            f"""\
-            private class Transformer
-            {I}: Visitation.AbstractTransformer<IEnumerable<Reporting.Error>>
-            {{
-            """
-        )
+        f"""\
+private class Transformer
+{I}: Visitation.AbstractTransformer<IEnumerable<Reporting.Error>>
+{{
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -1576,22 +1578,20 @@ def _generate_verify_enumeration(enumeration: intermediate.Enumeration) -> Strip
     name = csharp_naming.enum_name(enumeration.name)
 
     return Stripped(
-        textwrap.dedent(
-            f"""\
-            /// <summary>
-            /// Verify that <paramref name="that" /> is a valid enumeration value.
-            /// </summary>
-            public static IEnumerable<Reporting.Error> Verify{name}(
-            {I}Aas.{name} that)
-            {{
-            {I}if (!EnumValueSet.For{name}.Contains(
-            {II}(int)that))
-            {I}{{
-            {II}yield return new Reporting.Error(
-            {III}$"Invalid {name}: {{that}}");
-            {I}}}
-            }}"""
-        )
+        f"""\
+/// <summary>
+/// Verify that <paramref name="that" /> is a valid enumeration value.
+/// </summary>
+public static IEnumerable<Reporting.Error> Verify{name}(
+{I}Aas.{name} that)
+{{
+{I}if (!EnumValueSet.For{name}.Contains(
+{II}(int)that))
+{I}{{
+{II}yield return new Reporting.Error(
+{III}$"Invalid {name}: {{that}}");
+{I}}}
+}}"""
     )
 
 
@@ -1634,11 +1634,9 @@ def _generate_verify_constrained_primitive(
     if len(blocks) == 0:
         blocks.append(
             Stripped(
-                textwrap.dedent(
-                    """\
-                    // There is no verification specified.
-                    yield break;"""
-                )
+                """\
+// There is no verification specified.
+yield break;"""
             )
         )
 
@@ -1651,16 +1649,14 @@ def _generate_verify_constrained_primitive(
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            f"""\
-            /// <summary>
-            /// Verify the constraints of <paramref name="that" />.
-            /// </summary>
-            public static IEnumerable<Reporting.Error> Verify{name} (
-            {I}{that_type} that)
-            {{
-            """
-        )
+        f"""\
+/// <summary>
+/// Verify the constraints of <paramref name="that" />.
+/// </summary>
+public static IEnumerable<Reporting.Error> Verify{name} (
+{I}{that_type} that)
+{{
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -1750,11 +1746,9 @@ using Visitation = {namespace}.Visitation;"""
 
     verification_blocks.append(
         Stripped(
-            textwrap.dedent(
-                f"""\
-                private static readonly Verification.Transformer _transformer = (
-                {I}new Verification.Transformer());"""
-            )
+            f"""\
+private static readonly Verification.Transformer _transformer = (
+{I}new Verification.Transformer());"""
         )
     )
 
@@ -1771,22 +1765,20 @@ using Visitation = {namespace}.Visitation;"""
 
     verification_blocks.append(
         Stripped(
-            textwrap.dedent(
-                f"""\
-                /// <summary>
-                /// Verify the constraints of <paramref name="that" /> recursively.
-                /// </summary>
-                /// <param name="that">
-                /// The instance of the meta-model to be verified
-                /// </param>
-                public static IEnumerable<Reporting.Error> Verify(Aas.IClass that)
-                {{
-                {I}foreach (var error in _transformer.Transform(that))
-                {I}{{
-                {II}yield return error;
-                {I}}}
-                }}"""
-            )
+            f"""\
+/// <summary>
+/// Verify the constraints of <paramref name="that" /> recursively.
+/// </summary>
+/// <param name="that">
+/// The instance of the meta-model to be verified
+/// </param>
+public static IEnumerable<Reporting.Error> Verify(Aas.IClass that)
+{{
+{I}foreach (var error in _transformer.Transform(that))
+{I}{{
+{II}yield return error;
+{I}}}
+}}"""
         )
     )
 
@@ -1822,15 +1814,13 @@ using Visitation = {namespace}.Visitation;"""
 
     verification_writer = io.StringIO()
     verification_writer.write(
-        textwrap.dedent(
-            f"""\
-            namespace {namespace}
-            {{
-            {I}/// <summary>
-            {I}/// Verify that the instances of the meta-model satisfy the invariants.
-            {I}/// </summary>
-            """
-        )
+        f"""\
+namespace {namespace}
+{{
+{I}/// <summary>
+{I}/// Verify that the instances of the meta-model satisfy the invariants.
+{I}/// </summary>
+"""
     )
 
     # region Write an example usage
