@@ -42,16 +42,14 @@ def _generate_ivisitor(symbol_table: intermediate.SymbolTable) -> Stripped:
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Define the interface for a visitor which visits the instances of the model.
-            /// </summary>
-            public interface IVisitor
-            {
-                public void Visit(IClass that);
-            """
-        )
+        f"""\
+/// <summary>
+/// Define the interface for a visitor which visits the instances of the model.
+/// </summary>
+public interface IVisitor
+{{
+{I}public void Visit(IClass that);
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -66,34 +64,43 @@ def _generate_ivisitor(symbol_table: intermediate.SymbolTable) -> Stripped:
 
 def _generate_visitor_through(symbol_table: intermediate.SymbolTable) -> Stripped:
     """Generate the visitor that simply iterates over the instances."""
-    blocks = []  # type: List[Stripped]
+    blocks = [
+        Stripped(
+            f"""\
+public void Visit(IClass that)
+{{
+{I}that.Accept(this);
+}}"""
+        )
+    ]  # type: List[Stripped]
 
     for symbol in symbol_table.symbols:
         if isinstance(symbol, intermediate.Enumeration):
             continue
 
         elif isinstance(symbol, intermediate.ConstrainedPrimitive):
+            # Constrained primitives are modeled as their constrainees in C#,
+            # so we do not visit them.
             continue
 
-        elif isinstance(symbol, intermediate.Interface):
+        elif isinstance(symbol, intermediate.AbstractClass):
+            # Abstract classes are modeled as interfaces in C#, so we do not transform
+            # them.
             continue
 
-        elif isinstance(symbol, intermediate.Class):
+        elif isinstance(symbol, intermediate.ConcreteClass):
             cls_name = csharp_naming.class_name(symbol.name)
             blocks.append(
                 Stripped(
-                    textwrap.dedent(
-                        f"""\
-                public void Visit({cls_name} that)
-                {{
-                    // Just descend through, do nothing with the <c>that</c>
-                    foreach (var something in that.DescendOnce())
-                    {{
-                        Visit(something);
-                    }}
-                }}
-                """
-                    )
+                    f"""\
+public void Visit({cls_name} that)
+{{
+{I}// Just descend through, do nothing with <c>that</c>
+{I}foreach (var something in that.DescendOnce())
+{I}{{
+{II}Visit(something);
+{I}}}
+}}"""
                 )
             )
 
@@ -102,24 +109,72 @@ def _generate_visitor_through(symbol_table: intermediate.SymbolTable) -> Strippe
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Just descend through the instances without any action.
-            /// </summary>
-            /// <remarks>
-            /// This class is meaningless for itself. However, it is a good base if you
-            /// want to descend through instances and apply actions only on a subset of
-            /// classes.
-            /// </remarks>
-            public class VisitorThrough
-            {
-                public void Visit(IClass that)
-                {{
-                    that.Accept(this);
-                }}
-            """
+        """\
+/// <summary>
+/// Just descend through the instances without any action.
+/// </summary>
+/// <remarks>
+/// This class is meaningless for itself. However, it is a good base if you
+/// want to descend through instances and apply actions only on a subset of
+/// classes.
+/// </remarks>
+public class VisitorThrough : IVisitor
+{
+"""
+    )
+
+    for i, block in enumerate(blocks):
+        if i > 0:
+            writer.write("\n\n")
+        writer.write(textwrap.indent(block, I))
+
+    writer.write("\n}  // public class VisitorThrough")
+
+    return Stripped(writer.getvalue())
+
+
+def _generate_abstract_visitor(symbol_table: intermediate.SymbolTable) -> Stripped:
+    """Generate the visitor that performs double-dispatch."""
+    blocks = [
+        Stripped(
+            f"""\
+public void Visit(IClass that)
+{{
+{I}that.Accept(this);
+}}"""
         )
+    ]  # type: List[Stripped]
+
+    for symbol in symbol_table.symbols:
+        if isinstance(symbol, intermediate.Enumeration):
+            continue
+
+        elif isinstance(symbol, intermediate.ConstrainedPrimitive):
+            # Constrained primitives are modeled as their constrainees in C#,
+            # so we do not visit them.
+            continue
+
+        elif isinstance(symbol, intermediate.AbstractClass):
+            # Abstract classes are modeled as interfaces in C#, so we do not transform
+            # them.
+            continue
+
+        elif isinstance(symbol, intermediate.Class):
+            cls_name = csharp_naming.class_name(symbol.name)
+            blocks.append(Stripped(f"public abstract void Visit({cls_name} that);"))
+
+        else:
+            assert_never(symbol)
+
+    writer = io.StringIO()
+    writer.write(
+        """\
+/// <summary>
+/// Perform double-dispatch to visit the concrete instances.
+/// </summary>
+public abstract class AbstractVisitor : IVisitor
+{
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -127,7 +182,7 @@ def _generate_visitor_through(symbol_table: intermediate.SymbolTable) -> Strippe
             writer.write("\n")
         writer.write(textwrap.indent(block, I))
 
-    writer.write("\n}  // public class VisitorThrough")
+    writer.write("\n}  // public abstract class AbstractVisitor")
 
     return Stripped(writer.getvalue())
 
@@ -158,17 +213,15 @@ def _generate_ivisitor_with_context(symbol_table: intermediate.SymbolTable) -> S
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Define the interface for a visitor which visits the instances of the model.
-            /// </summary>
-            /// <typeparam name="C">Context type</typeparam>
-            public interface IVisitorWithContext<C>
-            {
-                public void Visit(IClass that, C context);
-            """
-        )
+        f"""\
+/// <summary>
+/// Define the interface for a visitor which visits the instances of the model.
+/// </summary>
+/// <typeparam name="C">Context type</typeparam>
+public interface IVisitorWithContext<C>
+{{
+{I}public void Visit(IClass that, C context);
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -181,6 +234,67 @@ def _generate_ivisitor_with_context(symbol_table: intermediate.SymbolTable) -> S
     return Stripped(writer.getvalue())
 
 
+def _generate_abstract_visitor_with_context(
+    symbol_table: intermediate.SymbolTable,
+) -> Stripped:
+    """Generate the visitor with context that performs double-dispatch."""
+    blocks = [
+        Stripped(
+            f"""\
+public void Visit(IClass that, C context)
+{{
+{I}that.Accept(this, context);
+}}"""
+        )
+    ]  # type: List[Stripped]
+
+    for symbol in symbol_table.symbols:
+        if isinstance(symbol, intermediate.Enumeration):
+            continue
+
+        elif isinstance(symbol, intermediate.ConstrainedPrimitive):
+            # Constrained primitives are modeled as their constrainees in C#,
+            # so we do not visit them.
+            continue
+
+        elif isinstance(symbol, intermediate.AbstractClass):
+            # Abstract classes are modeled as interfaces in C#, so we do not transform
+            # them.
+            continue
+
+        elif isinstance(symbol, intermediate.Class):
+            cls_name = csharp_naming.class_name(symbol.name)
+            blocks.append(
+                Stripped(f"public abstract void Visit({cls_name} that, C context);")
+            )
+
+        else:
+            assert_never(symbol)
+
+    writer = io.StringIO()
+    writer.write(
+        f"""\
+/// <summary>
+/// Perform double-dispatch to visit the concrete instances
+/// with context.
+/// </summary>
+/// <typeparam name="C">Context type</typeparam>
+public abstract class AbstractVisitorWithContext<C>
+{I}: IVisitorWithContext<C>
+{{
+"""
+    )
+
+    for i, block in enumerate(blocks):
+        if i > 0:
+            writer.write("\n")
+        writer.write(textwrap.indent(block, I))
+
+    writer.write("\n}  // public abstract class AbstractVisitorWithContext")
+
+    return Stripped(writer.getvalue())
+
+
 def _generate_itransformer(symbol_table: intermediate.SymbolTable) -> Stripped:
     """Generate the transformer interface."""
     blocks = []  # type: List[Stripped]
@@ -208,18 +322,16 @@ def _generate_itransformer(symbol_table: intermediate.SymbolTable) -> Stripped:
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Define the interface for a transformer which transforms recursively
-            /// the instances into something else.
-            /// </summary>
-            /// <typeparam name="T">The type of the transformation result</typeparam>
-            public interface ITransformer<T>
-            {
-                public T Transform(IClass that);
-            """
-        )
+        f"""\
+/// <summary>
+/// Define the interface for a transformer which transforms recursively
+/// the instances into something else.
+/// </summary>
+/// <typeparam name="T">The type of the transformation result</typeparam>
+public interface ITransformer<T>
+{{
+{I}public T Transform(IClass that);
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -236,13 +348,11 @@ def _generate_abstract_transformer(symbol_table: intermediate.SymbolTable) -> St
     """Generate the most abstract transformer that merely double-dispatches."""
     blocks = [
         Stripped(
-            textwrap.dedent(
-                f"""\
-                public T Transform(IClass that)
-                {{
-                {I}return that.Transform(this);
-                }}"""
-            )
+            f"""\
+public T Transform(IClass that)
+{{
+{I}return that.Transform(this);
+}}"""
         )
     ]  # type: List[Stripped]
 
@@ -269,17 +379,15 @@ def _generate_abstract_transformer(symbol_table: intermediate.SymbolTable) -> St
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Perform double-dispatch to transform recursively
-            /// the instances into something else.
-            /// </summary>
-            /// <typeparam name="T">The type of the transformation result</typeparam>
-            public abstract class AbstractTransformer<T> : ITransformer<T>
-            {
-            """
-        )
+        """\
+/// <summary>
+/// Perform double-dispatch to transform recursively
+/// the instances into something else.
+/// </summary>
+/// <typeparam name="T">The type of the transformation result</typeparam>
+public abstract class AbstractTransformer<T> : ITransformer<T>
+{
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -321,19 +429,17 @@ def _generate_itransformer_with_context(
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            """\
-            /// <summary>
-            /// Define the interface for a transformer which recursively transforms
-            /// the instances into something else while the context is passed along.
-            /// </summary>
-            /// <typeparam name="C">Type of the transformation context</typeparam>
-            /// <typeparam name="T">The type of the transformation result</typeparam>
-            public interface ITransformerWithContext<C, T>
-            {
-                public T Transform(IClass that, C context);
-            """
-        )
+        f"""\
+/// <summary>
+/// Define the interface for a transformer which recursively transforms
+/// the instances into something else while the context is passed along.
+/// </summary>
+/// <typeparam name="C">Type of the transformation context</typeparam>
+/// <typeparam name="T">The type of the transformation result</typeparam>
+public interface ITransformerWithContext<C, T>
+{{
+{I}public T Transform(IClass that, C context);
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -352,13 +458,11 @@ def _generate_abstract_transformer_with_context(
     """Generate the most abstract transformer that merely double-dispatches."""
     blocks = [
         Stripped(
-            textwrap.dedent(
-                f"""\
-                public T Transform(IClass that, C context)
-                {{
-                {I}return that.Transform(this, context);
-                }}"""
-            )
+            f"""\
+public T Transform(IClass that, C context)
+{{
+{I}return that.Transform(this, context);
+}}"""
         )
     ]  # type: List[Stripped]
 
@@ -387,19 +491,17 @@ def _generate_abstract_transformer_with_context(
 
     writer = io.StringIO()
     writer.write(
-        textwrap.dedent(
-            f"""\
-            /// <summary>
-            /// Perform double-dispatch to transform recursively
-            /// the instances into something else.
-            /// </summary>
-            /// <typeparam name="C">The type of the transformation context</typeparam>
-            /// <typeparam name="T">The type of the transformation result</typeparam>
-            public abstract class AbstractTransformerWithContext<C, T>
-            {I}: ITransformerWithContext<C, T>
-            {{
-            """
-        )
+        f"""\
+/// <summary>
+/// Perform double-dispatch to transform recursively
+/// the instances into something else.
+/// </summary>
+/// <typeparam name="C">The type of the transformation context</typeparam>
+/// <typeparam name="T">The type of the transformation result</typeparam>
+public abstract class AbstractTransformerWithContext<C, T>
+{I}: ITransformerWithContext<C, T>
+{{
+"""
     )
 
     for i, block in enumerate(blocks):
@@ -436,7 +538,10 @@ def generate(
 
     visitation_blocks = [
         _generate_ivisitor(symbol_table=symbol_table),
+        _generate_visitor_through(symbol_table=symbol_table),
+        _generate_abstract_visitor(symbol_table=symbol_table),
         _generate_ivisitor_with_context(symbol_table=symbol_table),
+        _generate_abstract_visitor_with_context(symbol_table=symbol_table),
         _generate_itransformer(symbol_table=symbol_table),
         _generate_abstract_transformer(symbol_table=symbol_table),
         _generate_itransformer_with_context(symbol_table=symbol_table),
