@@ -2,33 +2,11 @@
 
 import textwrap
 import unittest
-from typing import Tuple, Optional, MutableMapping, List
-
-from icontract import ensure
+from typing import Optional, MutableMapping
 
 import tests.common
-from aas_core_codegen import intermediate, infer_for_schema
-from aas_core_codegen.infer_for_schema import _len as infer_for_schema_len
-from aas_core_codegen.common import Identifier, Error
-
-
-@ensure(lambda result: (result[0] is not None) ^ (result[1] is not None))
-def infer_constraints_by_properties_of_class_something(
-    source: str,
-) -> Tuple[
-    Optional[MutableMapping[intermediate.Property, infer_for_schema_len.LenConstraint]],
-    Optional[List[Error]],
-]:
-    """Translate the ``source`` into inferred constraints of the class ``Something``."""
-    symbol_table, error = tests.common.translate_source_to_intermediate(source=source)
-    assert error is None, tests.common.most_underlying_messages(error)
-    assert symbol_table is not None
-    symbol = symbol_table.must_find(Identifier("Something"))
-    assert isinstance(symbol, intermediate.Class)
-
-    result = infer_for_schema_len.len_constraints_from_invariants(cls=symbol)
-
-    return result
+import tests.infer_for_schema.common
+from aas_core_codegen import infer_for_schema, intermediate
 
 
 class Test_expected(unittest.TestCase):
@@ -46,15 +24,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
-        self.assertEqual("{}", text)
+        text = infer_for_schema.dump(constraints_by_props)
+        self.assertEqual(
+            textwrap.dedent(
+                """\
+                ConstraintsByProperty(
+                  len_constraints_by_property={},
+                  patterns_by_property={})"""
+            ),
+            text,
+        )
 
     def test_min_value_constant_left(self) -> None:
         source = textwrap.dedent(
@@ -72,23 +61,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=11,
-                max_value=None)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=11,
+                      max_value=None)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -109,23 +101,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=11,
-                max_value=None)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=11,
+                      max_value=None)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -146,23 +141,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=None,
-                max_value=9)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=None,
+                      max_value=9)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -183,23 +181,70 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=None,
-                max_value=9)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=None,
+                      max_value=9)},
+                  patterns_by_property={})"""
+            ),
+            text,
+        )
+
+    def test_max_value_constant_right_and_not_required(self) -> None:
+        source = textwrap.dedent(
+            """\
+            @invariant(
+                lambda self:
+                not (self.some_property is not None)
+                or len(self.some_property) <= 128
+            )
+            class Something:
+                some_property: str
+
+                def __init__(self, some_property: str) -> None:
+                    self.some_property = some_property
+
+
+            __book_url__ = "dummy"
+            __book_version__ = "dummy"
+            """
+        )
+
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
+            source=source
+        )
+
+        constraints_by_props = constraints_by_class[something_cls]
+
+        text = infer_for_schema.dump(constraints_by_props)
+        self.assertEqual(
+            textwrap.dedent(
+                """\
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=None,
+                      max_value=128)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -220,23 +265,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=10,
-                max_value=10)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=10,
+                      max_value=10)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -257,23 +305,26 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=10,
-                max_value=10)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=10,
+                      max_value=10)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -298,28 +349,31 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
             source=source
         )
 
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
+        constraints_by_props = constraints_by_class[something_cls]
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-            {
-              'some_property':
-              LenConstraint(
-                min_value=10,
-                max_value=10)
-            }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=10,
+                      max_value=10)},
+                  patterns_by_property={})"""
             ),
             text,
         )
 
-    def test_inheritance(self) -> None:
+    def test_no_inheritance_by_default(self) -> None:
         source = textwrap.dedent(
             """\
             @invariant(lambda self: len(self.some_property) > 3)
@@ -344,28 +398,34 @@ class Test_expected(unittest.TestCase):
             """
         )
 
-        by_props, errors = infer_constraints_by_properties_of_class_something(
-            source=source
-        )
-
-        assert errors is None, tests.common.most_underlying_messages(errors)
-        assert by_props is not None
-
         # NOTE (mristin, 2022-01-02):
         # We infer only the constraints as specified in the class itself, and
         # ignore the constraints of the ancestors in *this particular kind of
         # inference*.
+        #
+        # This is necessary as we want to use these constraints to generate schemas
+        # whereas it is the job of the schema engine to stack the constraints together.
 
-        text = infer_for_schema.dump_len_constraints_by_properties(by_props)
+        (
+            _,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
+            source=source
+        )
+
+        constraints_by_props = constraints_by_class[something_cls]
+
+        text = infer_for_schema.dump(constraints_by_props)
         self.assertEqual(
             textwrap.dedent(
                 """\
-                {
-                  'some_property':
-                  LenConstraint(
-                    min_value=6,
-                    max_value=None)
-                }"""
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=6,
+                      max_value=None)},
+                  patterns_by_property={})"""
             ),
             text,
         )
@@ -389,13 +449,22 @@ class Test_unexpected(unittest.TestCase):
             """
         )
 
-        _, errors = infer_constraints_by_properties_of_class_something(source=source)
+        (
+            symbol_table,
+            _,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls(
+            source=source
+        )
 
-        assert errors is not None
+        _, error = infer_for_schema.infer_constraints_by_class(
+            symbol_table=symbol_table
+        )
+
+        assert error is not None
         self.assertEqual(
             "The property some_property has conflicting invariants on the length: "
             "the minimum length, 11, contradicts the maximum length 2.",
-            tests.common.most_underlying_messages(errors),
+            tests.common.most_underlying_messages(error),
         )
 
     def test_conflicting_min_and_exact(self) -> None:
@@ -415,13 +484,22 @@ class Test_unexpected(unittest.TestCase):
             """
         )
 
-        _, errors = infer_constraints_by_properties_of_class_something(source=source)
+        (
+            symbol_table,
+            _,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls(
+            source=source
+        )
 
-        assert errors is not None
+        _, error = infer_for_schema.infer_constraints_by_class(
+            symbol_table=symbol_table
+        )
+
+        assert error is not None
         self.assertEqual(
             "The property some_property has conflicting invariants on the length: "
             "the minimum length, 11, contradicts the exactly expected length 3.",
-            tests.common.most_underlying_messages(errors),
+            tests.common.most_underlying_messages(error),
         )
 
     def test_conflicting_max_and_exact(self) -> None:
@@ -441,13 +519,203 @@ class Test_unexpected(unittest.TestCase):
             """
         )
 
-        _, errors = infer_constraints_by_properties_of_class_something(source=source)
+        (
+            symbol_table,
+            _,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls(
+            source=source
+        )
 
-        assert errors is not None
+        _, error = infer_for_schema.infer_constraints_by_class(
+            symbol_table=symbol_table
+        )
+
+        assert error is not None
         self.assertEqual(
             "The property some_property has conflicting invariants on the length: "
             "the maximum length, 9, contradicts the exactly expected length 30.",
-            tests.common.most_underlying_messages(errors),
+            tests.common.most_underlying_messages(error),
+        )
+
+
+class Test_stacking(unittest.TestCase):
+    def test_no_inheritance_involved(self) -> None:
+        source = textwrap.dedent(
+            """\
+            @invariant(lambda self: len(self.some_property) < 10)
+            class Something:
+                some_property: str
+
+                def __init__(self, some_property: str) -> None:
+                    self.some_property = some_property
+
+
+            __book_url__ = "dummy"
+            __book_version__ = "dummy"
+            """
+        )
+
+        # NOTE (mristin, 2022-05-18):
+        # This definition here is necessary for mypy.
+        constraints_by_class: Optional[
+            MutableMapping[
+                intermediate.ClassUnion, infer_for_schema.ConstraintsByProperty
+            ]
+        ] = None
+
+        (
+            symbol_table,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
+            source=source
+        )
+
+        constraints_by_class, error = infer_for_schema.merge_constraints_with_ancestors(
+            symbol_table=symbol_table, constraints_by_class=constraints_by_class
+        )
+        assert error is None, tests.common.most_underlying_messages(error)
+        assert constraints_by_class is not None
+
+        constraints_by_props = constraints_by_class[something_cls]
+
+        text = infer_for_schema.dump(constraints_by_props)
+        self.assertEqual(
+            textwrap.dedent(
+                """\
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=None,
+                      max_value=9)},
+                  patterns_by_property={})"""
+            ),
+            text,
+        )
+
+    def test_inheritance_from_parent_with_no_patterns_of_own(self) -> None:
+        source = textwrap.dedent(
+            """\
+            @invariant(lambda self: len(self.some_property) > 3)
+            class Parent:
+                some_property: str
+
+                def __init__(self, some_property: str) -> None:
+                    self.some_property = some_property
+
+
+            class Something(Parent):
+                def __init__(self, some_property: str) -> None:
+                    Parent.__init__(
+                        self,
+                        some_property=some_property
+                    )
+
+
+            __book_url__ = "dummy"
+            __book_version__ = "dummy"
+            """
+        )
+
+        # NOTE (mristin, 2022-05-18):
+        # This definition here is necessary for mypy.
+        constraints_by_class: Optional[
+            MutableMapping[
+                intermediate.ClassUnion, infer_for_schema.ConstraintsByProperty
+            ]
+        ] = None
+
+        (
+            symbol_table,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
+            source=source
+        )
+
+        constraints_by_class, error = infer_for_schema.merge_constraints_with_ancestors(
+            symbol_table=symbol_table, constraints_by_class=constraints_by_class
+        )
+        assert error is None, tests.common.most_underlying_messages(error)
+        assert constraints_by_class is not None
+
+        constraints_by_props = constraints_by_class[something_cls]
+
+        text = infer_for_schema.dump(constraints_by_props)
+        self.assertEqual(
+            textwrap.dedent(
+                """\
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=4,
+                      max_value=None)},
+                  patterns_by_property={})"""
+            ),
+            text,
+        )
+
+    def test_merge_with_parent(self) -> None:
+        source = textwrap.dedent(
+            """\
+            @invariant(lambda self: len(self.some_property) > 3)
+            class Parent:
+                some_property: str
+
+                def __init__(self, some_property: str) -> None:
+                    self.some_property = some_property
+
+
+            @invariant(lambda self: len(self.some_property) < 10)
+            class Something(Parent):
+                def __init__(self, some_property: str) -> None:
+                    Parent.__init__(
+                        self,
+                        some_property=some_property
+                    )
+
+
+            __book_url__ = "dummy"
+            __book_version__ = "dummy"
+            """
+        )
+
+        # NOTE (mristin, 2022-05-18):
+        # This definition here is necessary for mypy.
+        constraints_by_class: Optional[
+            MutableMapping[
+                intermediate.ClassUnion, infer_for_schema.ConstraintsByProperty
+            ]
+        ] = None
+
+        (
+            symbol_table,
+            something_cls,
+            constraints_by_class,
+        ) = tests.infer_for_schema.common.parse_to_symbol_table_and_something_cls_and_constraints_by_class(
+            source=source
+        )
+
+        constraints_by_class, error = infer_for_schema.merge_constraints_with_ancestors(
+            symbol_table=symbol_table, constraints_by_class=constraints_by_class
+        )
+        assert error is None, tests.common.most_underlying_messages(error)
+        assert constraints_by_class is not None
+
+        constraints_by_props = constraints_by_class[something_cls]
+
+        text = infer_for_schema.dump(constraints_by_props)
+        self.assertEqual(
+            textwrap.dedent(
+                """\
+                ConstraintsByProperty(
+                  len_constraints_by_property={
+                    'some_property': LenConstraint(
+                      min_value=4,
+                      max_value=9)},
+                  patterns_by_property={})"""
+            ),
+            text,
         )
 
 
